@@ -10,27 +10,32 @@
 namespace qc {
 
     class CompoundOperation final: public Operation {
-    protected:
+    private:
         std::vector<std::unique_ptr<Operation>> ops{};
 
     public:
-        explicit CompoundOperation(dd::QubitCount nq) {
-            std::strcpy(name, "Compound operation:");
+        explicit CompoundOperation(const std::size_t nq) {
+            name    = "Compound operation:";
             nqubits = nq;
             type    = Compound;
         }
 
-        [[nodiscard]] std::unique_ptr<Operation> clone() const override {
-            auto cloned_co = std::make_unique<CompoundOperation>(nqubits);
-            cloned_co->reserve(ops.size());
-
-            for (auto& op: ops) {
-                cloned_co->ops.emplace_back<>(op->clone());
-            }
-            return cloned_co;
+        explicit CompoundOperation(const std::size_t nq, std::vector<std::unique_ptr<Operation>>&& operations):
+            CompoundOperation(nq) {
+            ops = std::move(operations); // NOLINT (cppcoreguidelines-prefer-member-initializer)
         }
 
-        void setNqubits(dd::QubitCount nq) override {
+        [[nodiscard]] std::unique_ptr<Operation> clone() const override {
+            auto clonedCo = std::make_unique<CompoundOperation>(nqubits);
+            clonedCo->reserve(ops.size());
+
+            for (const auto& op: ops) {
+                clonedCo->ops.emplace_back<>(op->clone());
+            }
+            return clonedCo;
+        }
+
+        void setNqubits(const std::size_t nq) override {
             nqubits = nq;
             for (auto& op: ops) {
                 op->setNqubits(nq);
@@ -45,6 +50,10 @@ namespace qc {
             return std::any_of(ops.cbegin(), ops.cend(), [](const auto& op) { return op->isNonUnitaryOperation(); });
         }
 
+        [[nodiscard]] inline bool isSymbolicOperation() const override {
+            return std::any_of(ops.begin(), ops.end(), [](const auto& op) { return op->isSymbolicOperation(); });
+        }
+
         [[nodiscard]] bool equals(const Operation& op, const Permutation& perm1, const Permutation& perm2) const override {
             if (const auto* comp = dynamic_cast<const CompoundOperation*>(&op)) {
                 if (comp->ops.size() != ops.size()) {
@@ -59,9 +68,8 @@ namespace qc {
                     ++it;
                 }
                 return true;
-            } else {
-                return false;
             }
+            return false;
         }
         [[nodiscard]] bool equals(const Operation& operation) const override {
             return equals(operation, {}, {});
@@ -89,25 +97,25 @@ namespace qc {
             return os;
         }
 
-        [[nodiscard]] bool actsOn(dd::Qubit i) const override {
+        [[nodiscard]] bool actsOn(const Qubit i) const override {
             return std::any_of(ops.cbegin(), ops.cend(), [&i](const auto& op) { return op->actsOn(i); });
         }
 
+        void addDepthContribution(std::vector<std::size_t>& depths) const override {
+            for (const auto& op: ops) {
+                op->addDepthContribution(depths);
+            }
+        }
+
         void dumpOpenQASM(std::ostream& of, const RegisterNames& qreg, const RegisterNames& creg) const override {
             for (const auto& op: ops) {
                 op->dumpOpenQASM(of, qreg, creg);
             }
         }
 
-        void dumpQiskit(std::ostream& of, const RegisterNames& qreg, const RegisterNames& creg, const char* anc_reg_name) const override {
-            for (const auto& op: ops) {
-                op->dumpQiskit(of, qreg, creg, anc_reg_name);
-            }
-        }
-
         /**
-		 * Pass-Through
-		 */
+                 * Pass-Through
+                 */
 
         // Iterators (pass-through)
         auto               begin() noexcept { return ops.begin(); }
@@ -126,24 +134,24 @@ namespace qc {
         // Capacity (pass-through)
         [[nodiscard]] bool        empty() const noexcept { return ops.empty(); }
         [[nodiscard]] std::size_t size() const noexcept { return ops.size(); }
-        [[nodiscard]] std::size_t max_size() const noexcept { return ops.max_size(); }
+        [[nodiscard]] std::size_t max_size() const noexcept { return ops.max_size(); } // NOLINT (readability-identifier-naming)
         [[nodiscard]] std::size_t capacity() const noexcept { return ops.capacity(); }
 
-        void reserve(std::size_t new_cap) { ops.reserve(new_cap); }
-        void shrink_to_fit() { ops.shrink_to_fit(); }
+        void reserve(std::size_t newCap) { ops.reserve(newCap); }
+        void shrink_to_fit() { ops.shrink_to_fit(); } // NOLINT (readability-identifier-naming)
 
         // Modifiers (pass-through)
         void                                              clear() noexcept { ops.clear(); }
-        void                                              pop_back() { return ops.pop_back(); }
+        void                                              pop_back() { return ops.pop_back(); } // NOLINT (readability-identifier-naming)
         void                                              resize(std::size_t count) { ops.resize(count); }
         std::vector<std::unique_ptr<Operation>>::iterator erase(std::vector<std::unique_ptr<Operation>>::const_iterator pos) { return ops.erase(pos); }
         std::vector<std::unique_ptr<Operation>>::iterator erase(std::vector<std::unique_ptr<Operation>>::const_iterator first, std::vector<std::unique_ptr<Operation>>::const_iterator last) { return ops.erase(first, last); }
         template<class T>
-        void emplace_back(std::unique_ptr<T>& op) {
+        void emplace_back(std::unique_ptr<T>& op) { // NOLINT (readability-identifier-naming)
             ops.emplace_back(std::move(op));
         }
         template<class T, class... Args>
-        void emplace_back(Args&&... args) {
+        void emplace_back(Args&&... args) { // NOLINT (readability-identifier-naming)
             ops.emplace_back(std::make_unique<T>(args...));
         }
         template<class T, class... Args>
@@ -159,8 +167,8 @@ namespace qc {
 
         std::vector<std::unique_ptr<Operation>>& getOps() { return ops; }
 
-        [[nodiscard]] std::set<dd::Qubit> getUsedQubits() const override {
-            std::set<dd::Qubit> usedQubits{};
+        [[nodiscard]] std::set<Qubit> getUsedQubits() const override {
+            std::set<Qubit> usedQubits{};
             for (const auto& op: ops) {
                 usedQubits.merge(op->getUsedQubits());
             }
