@@ -1,7 +1,8 @@
-#
-# This file is part of MQT QMAP library which is released under the MIT license.
-# See file README.md or go to http://iic.jku.at/eda/research/quantum_verification/ for more information.
-#
+"""MQT QMAP library.
+
+This file is part of the MQT QMAP library released under the MIT license.
+See README.md or go to https://github.com/cda-tum/qmap for more information.
+"""
 
 import os
 import sys
@@ -15,10 +16,12 @@ if sys.platform == "win32" and sys.version_info > (3, 8, 0) and "Z3_ROOT" in os.
     if bin_path.exists():
         os.add_dll_directory(str(bin_path))
 
+from mqt.qmap.clifford_synthesis import optimize_clifford, synthesize_clifford
 from mqt.qmap.compile import compile
 from mqt.qmap.pyqmap import (
     Arch,
     Architecture,
+    CliffordSynthesizer,
     CommanderGrouping,
     Configuration,
     Encoding,
@@ -26,7 +29,13 @@ from mqt.qmap.pyqmap import (
     Layering,
     MappingResults,
     Method,
+    QuantumComputation,
     SwapReduction,
+    SynthesisConfiguration,
+    SynthesisResults,
+    Tableau,
+    TargetMetric,
+    Verbosity,
 )
 from mqt.qmap.subarchitectures import SubarchitectureOrder
 
@@ -43,4 +52,13 @@ __all__ = [
     "MappingResults",
     "Architecture",
     "SubarchitectureOrder",
+    "SynthesisConfiguration",
+    "SynthesisResults",
+    "TargetMetric",
+    "QuantumComputation",
+    "Tableau",
+    "CliffordSynthesizer",
+    "synthesize_clifford",
+    "optimize_clifford",
+    "Verbosity",
 ]
